from .engine import RaptorQEngine
from .raptorq_cpp_engine import RaptorQCppEngine


class Encoder:

    def __init__(
            self,
            data: bytes,
            symbol_size,
            raptorq_engine: RaptorQEngine = None
    ):
        self.data = data
        self.symbol_size = symbol_size
        if raptorq_engine is None:
            raptorq_engine = RaptorQCppEngine.default()
        self.raptorq_engine = raptorq_engine
        self.encoder = self.raptorq_engine.get_encoder(
            data,
            symbol_size,
        )

    def gen_symbol(self, symbol_id):
        return self.raptorq_engine.gen_symbol(self.encoder, symbol_id)

    def __del__(self):
        self.raptorq_engine.destroy_encoder(self.encoder)



import base64
import ctypes
import typing

from .engine import RaptorQEngine


class RaptorQCppEngine(RaptorQEngine):

    def __init__(self, path_to_lib: str):
        self.lib = ctypes.CDLL(path_to_lib)
        assert self.lib.test_link(1, 2) == 3, 'Invalid library'

        # Define signatures
        self.lib.get_decoder.argtypes = [ctypes.c_int, ctypes.c_int, ctypes.c_int]
        self.lib.get_decoder.restype = ctypes.c_void_p

        self.lib.may_try_decode.argtypes = [ctypes.c_void_p]
        self.lib.may_try_decode.restype = ctypes.c_bool

        self.lib.add_symbol.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int, ctypes.c_char_p]
        self.lib.add_symbol.restype = ctypes.c_bool

        self.lib.try_decode.argtypes = [ctypes.c_void_p]
        self.lib.try_decode.restype = ctypes.c_char_p

        self.lib.destroy_decoder.argtypes = [ctypes.c_void_p]
        self.lib.destroy_decoder.restype = None

        self.lib.get_encoder.argtypes = [ctypes.c_char_p, ctypes.c_int, ctypes.c_int]
        self.lib.get_encoder.restype = ctypes.c_void_p

        self.lib.gen_symbol.argtypes = [ctypes.c_void_p, ctypes.c_int]
        self.lib.gen_symbol.restype = ctypes.c_char_p

        self.lib.destroy_encoder.argtypes = [ctypes.c_void_p]
        self.lib.destroy_encoder.restype = None

    def get_decoder(self, symbol_count: int, symbol_size: int, data_size: int):
        return self.lib.get_decoder(symbol_count, symbol_size, data_size)

    def may_try_decode(self, decoder) -> bool:
        return self.lib.may_try_decode(decoder)

    def add_symbol(self, decoder, symbol_id: int, symbol_data: bytes) -> bool:
        return self.lib.add_symbol(decoder, symbol_id, len(symbol_data), symbol_data)

    def try_decode(self, decoder) -> typing.Optional[bytes]:
        res = self.lib.try_decode(decoder)
        if res is not None:
            res = base64.b64decode(res)
        return res

    def destroy_decoder(self, decoder):
        self.lib.destroy_decoder(decoder)

    def get_encoder(self, data: bytes, symbol_size: int):
        res = self.lib.get_encoder(data, len(data), symbol_size)
        if res is None:
            raise Exception('Failed to create encoder')
        return res

    def gen_symbol(self, encoder, symbol_id: int) -> bytes:
        res = self.lib.gen_symbol(encoder, symbol_id)
        if res is None:
            raise Exception('Failed to generate symbol')
        return base64.b64decode(res)

    def destroy_encoder(self, encoder):
        self.lib.destroy_encoder(encoder)

    @classmethod
    def default(cls):
        import pkg_resources
        import platform

        arch = platform.system().lower()
        machine = platform.machine().lower()
        name = f'libpyraptorq.{machine}'
        if arch == 'darwin':
            name += '.dylib'
        elif arch == 'linux':
            name += '.so'
        elif arch == 'windows':
            name += '.dll'
        else:
            raise Exception('Unsupported platform')

        if not pkg_resources.resource_exists('pyraptorq', f'distlib/{arch}/{name}'):
            raise Exception('No binaries found')
        return cls(pkg_resources.resource_filename('pyraptorq', f'distlib/{arch}/{name}'))



from .engine import RaptorQEngine
from .raptorq_cpp_engine import RaptorQCppEngine


class Decoder:

    def __init__(
            self,
            symbol_count,
            symbol_size,
            data_size,
            raptorq_engine: RaptorQEngine = None
    ):
        self.symbol_count = symbol_count
        self.symbol_size = symbol_size
        self.data_size = data_size
        if raptorq_engine is None:
            raptorq_engine = RaptorQCppEngine.default()
        self.raptorq_engine = raptorq_engine
        self.decoder = self.raptorq_engine.get_decoder(
            symbol_count,
            symbol_size,
            data_size
        )

    def add_symbol(self, symbol_id, symbol_data):
        return self.raptorq_engine.add_symbol(
            self.decoder,
            symbol_id,
            symbol_data
        )

    def may_try_decode(self):
        return self.raptorq_engine.may_try_decode(self.decoder)

    def try_decode(self):
        return self.raptorq_engine.try_decode(self.decoder)

    def __del__(self):
        self.raptorq_engine.destroy_decoder(self.decoder)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyraptorq",
    version="0.1.5",
    author="Maksim Kurbatov",
    author_email="cyrbatoff@gmail.com",
    description="RaptorQ RFC6330 C++ bindings for python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages('.', exclude=['.idea', 'tests', 'examples']),
    package_data={'pyraptorq': ['distlib/**/*']},
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Software Development :: Libraries",
    ],
    url="https://github.com/yungwine/pyraptorq",
    python_requires='>=3.9',
    py_modules=["pyraptorq"]
)



import typing
from abc import ABC, abstractmethod


class RaptorQEngine(ABC):

    @abstractmethod
    def get_decoder(self, symbol_count: int, symbol_size: int, data_size: int):
        ...

    @abstractmethod
    def may_try_decode(self, decoder) -> bool:
        ...

    @abstractmethod
    def add_symbol(self, decoder, symbol_id: int, symbol_data: bytes) -> bool:
        ...

    @abstractmethod
    def try_decode(self, decoder) -> typing.Optional[bytes]:
        ...

    @abstractmethod
    def destroy_decoder(self, decoder):
        ...

    @abstractmethod
    def get_encoder(self, data: bytes, symbol_size: int):
        ...

    @abstractmethod
    def gen_symbol(self, encoder, symbol_id: int) -> bytes:
        ...

    @abstractmethod
    def destroy_encoder(self, encoder):
        ...
